@@ -159,7 +159,8 @@ sub update {
 	) or die "meeting not found: ".$meeting_id;
 
     my ($users, $groups, $guests) = $self->_collate_participants;
-    # underlying adapter does not support groups or guests
+    # underlying adapter does not yet support groups or guests as
+    # participants.
 
     $self->_massage_participants ($users, $groups, $guests);
     #
@@ -344,20 +345,6 @@ to retrieve on a meeting id.
 
 sub list {return shift->_not_available}
 
-=head1 SEE ALSO
-
-=over 4
-
-=item Elive::Entity::Meeting
-
-=item Elive::Entity::ParticipantList::Participants
-
-=item Elive::Entity::ParticipantList::Participant
-
-=back
-
-=cut
-
 sub _thaw {
     my ($class, $db_data, @args) = @_;
 
@@ -396,4 +383,16 @@ sub _thaw {
     return $class->SUPER::_thaw($db_data, @args);
 }
 
+=head1 SEE ALSO
+
+L<Elive::Entity::Meeting>
+
+L<Elive::View::Session>
+
+L<Elive::Entity::ParticipantList::Participants>
+
+L<Elive::Entity::ParticipantList::Participant>
+
+=cut
+
 1;
