@@ -12,7 +12,7 @@ A partial emulation of the SOAP connection and database backend.
 =cut
 
 use Elive::Connection::SDK;
-use base 'Elive::Connection::SDK';
+use parent 'Elive::Connection::SDK';
 
 use Elive;
 use Elive::Entity;
