@@ -51,17 +51,18 @@ sub _pack_data {
 
 	my $property = $property_types->{$prop};
 
-	my ($type, $is_array, $is_struct, $_is_ref, $class) = Elive::Util::parse_type( $property );
+	my $type_info = Elive::Util::inspect_type( $property );
+	my $type = $type_info->type;
 
 	for ($db_data{$prop}) {
 	    die "$class undefined property $prop"
 		unless defined;
 
-	    $_ = [split($class->separator, $_)]
-		if ($is_array && !ref);
+	    $_ = [split($type->separator, $_)]
+		if ($type_info->is_array && !ref);
 
-	    if ($is_struct) {
-		my ($adapter, $packed_data) = _pack_data($type, $_);
+	    if ($type_info->is_struct) {
+		my ($adapter, $packed_data) = _pack_data($type_info->elemental_type, $_);
 		$_ = {$adapter => $packed_data};
 	    }
 	}
